package com.tzld.piaoquan.ad.engine.service.predict.model.threshold;

import com.tzld.piaoquan.ad.engine.commons.redis.AlgorithmRedisHelper;
import com.tzld.piaoquan.ad.engine.service.predict.config.AdOutV1OnlineWeightConfig;
import com.tzld.piaoquan.ad.engine.service.predict.constant.RuleRedisKeyConst;
import com.tzld.piaoquan.ad.engine.service.predict.containner.ThresholdModelContainer;
import com.tzld.piaoquan.ad.engine.service.predict.param.RuleParamHelper;
import com.tzld.piaoquan.ad.engine.service.predict.param.ThresholdPredictModelParam;
import org.springframework.beans.factory.annotation.Autowired;

import java.text.SimpleDateFormat;
import java.util.*;

public class ScoreThresholdPredictModel extends ThresholdPredictModel{

    @Autowired
    RuleParamHelper paramHelper;

    @Autowired
    AlgorithmRedisHelper redisHelper;

    @Override
    String initName() {
        return "model";
    }
    @Override
    public Map<String, Object> predict(ThresholdPredictModelParam modelParam) {
        Map<String, Object> result;

        String modelKey = (String) modelParam.getAbtestParam().getOrDefault("model_key", "ad_out_v1");
        String userKeyName = RuleRedisKeyConst.KEY_NAME_PREFIX_AD_OUT_MODEL_SCORE_USER + modelKey + ":" + modelParam.getMid();
        String itemKeyName = RuleRedisKeyConst.KEY_NAME_PREFIX_AD_OUT_MODEL_SCORE_ITEM + modelKey + ":" + modelParam.getVideoId();

        String userScore = redisHelper.get(userKeyName);
        String itemScore = redisHelper.get(itemKeyName);

        if (userScore == null || itemScore == null) {
            // If offline scores are empty, fall back to baseline logic
            result = ThresholdModelContainer.
                    getBasicPredictModel()
                    .predict(modelParam);
            return result;
        }
        String configKeyPrefix = RuleRedisKeyConst.KEY_NAME_PREFIX_AD_OUT_MODEL_CONFIG
                + modelKey + ":" + modelParam.getAbtestParam().get("abtest_id") + ":"
                + modelParam.getAbtestParam().get("abtest_config_tag");
        String thresholdKey = configKeyPrefix + ":threshold";
        double offlineScore = Double.parseDouble(userScore) + Double.parseDouble(itemScore);

        SimpleDateFormat hourFormat = new SimpleDateFormat("HH");
        SimpleDateFormat weekDayFormat = new SimpleDateFormat("u");


        Map<String, String> onlineFeatures = new HashMap<>();
        onlineFeatures.put("ctx_apptype", modelParam.getAppType().toString());
        onlineFeatures.put("ctx_week", weekDayFormat.format(modelParam.getDate()));
        onlineFeatures.put("ctx_hour", hourFormat.format(modelParam.getDate()));

        double onlineScore=getOlineScore(onlineFeatures);
        double finalScore =getFinalScore(onlineScore,offlineScore);

        String thresholdValue = redisHelper.get(thresholdKey);
        double threshold = (thresholdValue != null) ? Double.parseDouble(thresholdValue) : 0.0;

        int adPredict;
        if (finalScore < threshold) {
            // If final score is below threshold, show the ad
            adPredict = 2;
        } else {
            // Otherwise, do not show the ad
            adPredict = 1;
        }
        result=new HashMap<>();
        result.put("user_score", userScore);
        result.put("item_score", itemScore);
        result.put("final_score", finalScore);
        result.put("online_score", onlineScore);
        result.put("threshold", threshold);
        result.put("ad_predict", adPredict);
        result.put("online_features", onlineFeatures);
        return result;
    }

    private double getFinalScore(Double onlineScore,Double offlineScore) {
        double finalScore = onlineScore + offlineScore;
        return 1.0 / (1.0 + Math.exp(0d-finalScore));
    }

    private double getOlineScore(Map<String, String> onlineFeatures ) {
        double score = AdOutV1OnlineWeightConfig.getWeight("bias");
        for (Map.Entry<String, String> entry : onlineFeatures.entrySet()) {
            double featureScore = AdOutV1OnlineWeightConfig.getWeight(entry.getKey() + "#" +  entry.getValue());
            score += featureScore;
        }
        return score;
    }
}
